﻿using Raylib_cs;
using NAudio.Wave;
using System.Numerics;
using FftSharp;

static class Program
{
    static WasapiLoopbackCapture? capture;
    static double[]? lastBuffer;
    static bool decibel = false;
    static Font font = Raylib.GetFontDefault();    
    const int WIDTH = 800;
    const int HEIGHT = 600;
    const int SAMPLE_RATE = 44100;
    const int BITS_PER_SAMPLE = 16;
    const int CHANNELS = 2;


    public static void Main()
    {
        Raylib.InitWindow(WIDTH, HEIGHT, "Spectrum");

        capture = new WasapiLoopbackCapture();
        capture.WaveFormat = new WaveFormat(SAMPLE_RATE, BITS_PER_SAMPLE, CHANNELS);

        capture.DataAvailable += (s, e) =>
        {
            int bytesPerSample = capture.WaveFormat.BitsPerSample / 8;
            int samplesRecorded = e.BytesRecorded / bytesPerSample;
            if (lastBuffer is null || lastBuffer.Length != samplesRecorded)
                lastBuffer = new double[samplesRecorded];
            for (int i = 0; i < samplesRecorded; i++)
                lastBuffer[i] = (double)BitConverter.ToInt16(e.Buffer, i * bytesPerSample);
        };

        capture.RecordingStopped += (s, e) =>
        {
            capture.Dispose();
        };

        capture.StartRecording();

        while (!Raylib.WindowShouldClose())
        {
            if (lastBuffer is null || lastBuffer.Length < 16)
            {
                Console.WriteLine("No buffer available");
            }
            else
            {
                Raylib.BeginDrawing();
                Raylib.ClearBackground(Raylib_cs.Color.WHITE);
                FftSharp.Windows.Hanning window = new FftSharp.Windows.Hanning();
                double[] windowed = window.Apply(lastBuffer);
                double[] zeroPadded = Pad.ZeroPad(windowed);
                double[] fftPower = decibel ?
                    FftSharp.Transform.FFTpower(zeroPadded) :
                    FftSharp.Transform.FFTmagnitude(zeroPadded);
                double[] fftFreq = FftSharp.Transform.FFTfreq(SAMPLE_RATE, fftPower.Length);

                double peakFreq = 0;
                double peakPower = 0;
                for (int i = 0; i < fftPower.Length; i++)
                {
                    if (fftPower[i] > peakPower)
                    {
                        peakPower = fftPower[i];
                        peakFreq = fftFreq[i];
                    }
                }

                // Due to the nquist frequency, we use only the half array
                int half_array = fftFreq.Length / 2;
                int offset = half_array / WIDTH;

                for (int index = 0, offsetDrawFreq = 1, x = 0; index < half_array - offset; index+=offset, offsetDrawFreq++, x++)
                {
                    double f = fftFreq[index];
                    /*                         x                   y                     w             h*/
                    Raylib.DrawLine(x, HEIGHT - (int)fftPower[index], x + 1, HEIGHT - (int)fftPower[index + offset], Raylib_cs.Color.RED);
                    if (offsetDrawFreq % /* valeur arbitraire */ 50 == 0)
                    {
                        Raylib.DrawTextPro(font,
                            String.Format($"{((int)f * 2)}Hz"),
                            new Vector2(x, HEIGHT - 100),
                            new Vector2(0, 0),
                            270f,
                            16f,
                            5f,
                            Raylib_cs.Color.BLACK
                            );
                    }
                }
                Raylib.EndDrawing();
            }
        }

        capture?.StopRecording();

        Raylib.CloseWindow();
    }
}
